@@ -141,7 +141,7 @@ def main():
     parser_help.set_defaults(func=help, parser=parser)
 
     # ADDCDROM
-    
+
     function = commands.addcdrom
     parser_addcdrom = subparsers.add_parser(
         "addcdrom",
@@ -151,8 +151,22 @@ def main():
     )
     parser_addcdrom.set_defaults(func=function)
 
+    # ADDGROUP
+
+    function = commands.addgroup
+    parser_addgroup = subparsers.add_parser(
+        "addgroup",
+        aliases=["add-group"],
+        parents=[parser_teach],
+        description=function.__doc__,
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+    )
+    parser_addgroup.add_argument("username")
+    parser_addgroup.add_argument("group")
+    parser_addgroup.set_defaults(func=function)
+
     # ADDKEY
-    
+
     function = commands.addkey
     parser_addkey = subparsers.add_parser(
         "addkey",
@@ -165,7 +179,7 @@ def main():
     parser_addkey.set_defaults(func=function)
 
     # ADDREPO
-    
+
     function = commands.addrepo
     parser_addrepo = subparsers.add_parser(
         "addrepo",
@@ -177,7 +191,7 @@ def main():
     parser_addrepo.set_defaults(func=function)
 
     # ADDUSER
-    
+
     function = commands.adduser
     parser_adduser = subparsers.add_parser(
         "adduser",
@@ -293,7 +307,7 @@ def main():
     parser_dailyupgrade.set_defaults(func=function)
 
     # DELUSER
-    
+
     function = commands.deluser
     parser_deluser = subparsers.add_parser(
         "deluser",
@@ -332,6 +346,19 @@ def main():
     )
     parser_describenew.set_defaults(func=function)
 
+    # DISABLE
+
+    function = commands.disable
+    parser_disable = subparsers.add_parser(
+        "disable",
+        aliases=["disuser", "lock"],
+        parents=[parser_teach],
+        description=function.__doc__,
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+    )
+    parser_disable.add_argument("username", nargs="+")
+    parser_disable.set_defaults(func=function)
+
     function = commands.distupgrade
     parser_distupgrade = subparsers.add_parser(
         "distupgrade",
@@ -364,6 +391,19 @@ def main():
     )
     parser_editsources.set_defaults(func=function)
 
+    # ENABLE
+
+    function = commands.enable
+    parser_enable = subparsers.add_parser(
+        "enable",
+        aliases=["enuser", "unlock"],
+        parents=[parser_teach],
+        description=function.__doc__,
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+    )
+    parser_enable.add_argument("username", nargs="+")
+    parser_enable.set_defaults(func=function)
+
     function = commands.extract
     parser_extract = subparsers.add_parser(
         "extract",
@@ -523,6 +563,15 @@ def main():
     )
     parser_listdaemons.set_defaults(func=function)
 
+    function = commands.listgroups
+    parser_listgroups = subparsers.add_parser(
+        "listgroups",
+        aliases=["list-groups", "groups"],
+        parents=[parser_teach],
+        description=function.__doc__,
+    )
+    parser_listgroups.set_defaults(func=function)
+
     function = commands.listfiles
     parser_listfiles = subparsers.add_parser(
         "listfiles",
@@ -537,6 +586,7 @@ def main():
     parser_listhold = subparsers.add_parser(
         "listhold",
         aliases=["list-hold"],
+        parents=[parser_teach],
         description=function.__doc__,
     )
     parser_listhold.set_defaults(func=function)
@@ -836,7 +886,7 @@ def main():
     parser_reportbug.set_defaults(func=function)
 
     # REPOS - list ppa repositories
-    
+
     function = commands.repos
     parser_repos = subparsers.add_parser(
         "repos",
@@ -854,8 +904,22 @@ def main():
     parser_restart.add_argument("daemon")
     parser_restart.set_defaults(func=function)
 
+    # RMGROUP
+
+    function = commands.rmgroup
+    parser_rmgroup = subparsers.add_parser(
+        "rmgroup",
+        aliases=["remove-group", "delgroup"],
+        parents=[parser_teach],
+        description=function.__doc__,
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+    )
+    parser_rmgroup.add_argument("username")
+    parser_rmgroup.add_argument("group")
+    parser_rmgroup.set_defaults(func=function)
+
     # RMREPO
-    
+
     function = commands.rmrepo
     parser_rmrepo = subparsers.add_parser(
         "rmrepo",
