@@ -6,7 +6,6 @@
 
 import os
 import re
-import sys
 import string
 import random
 import inspect
@@ -33,7 +32,23 @@ NO_UPGRADES = 'No packages known to be upgradable. Run "wajig update" to update
 
 def addcdrom(args):
     """Add a Debian CD/DVD to APT's list of available sources"""
-    perform.execute("/usr/bin/apt-cdrom add", root=True, teach=args.teach, noop=args.noop)
+    perform.execute("/usr/bin/apt-cdrom add",
+                    root=True, teach=args.teach, noop=args.noop)
+
+
+# ADDGROUP
+
+def addgroup(args):
+    """Add a user to a group.
+
+    Unix groups are used to provide group access to commands or
+    files. For example, adding users to the group `sudo` give those
+    users super user access through the `sudo` command.
+
+    $ wajig addgroup fred sudo"""
+    cmd = f'/usr/sbin/adduser {args.username} {args.group}'
+    perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
+
 
 # ADDKEY
 
@@ -52,6 +67,7 @@ def addkey(args):
     cmd += args.key
     perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
 
+
 # ADDREPO
 
 def addrepo(args):
@@ -63,23 +79,24 @@ def addrepo(args):
     $ wajig addrepo ppa:chromium-daily
     """
     util.requires_package("add-apt-repository")
-    perform.execute("/usr/bin/add-apt-repository " + args.ppa, root=True, teach=args.teach, noop=args.noop)
+    cmd = "/usr/bin/add-apt-repository " + args.ppa
+    perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
+
 
 # ADDUSER
 
 def adduser(args):
     """Add new user, multiple users, or as listed in a file.
 
-With just a username create a new password while adding user and 
-return that.
+    With just a username create a new password while adding user and
+    return that.
 
-  $ wajig adduser fred
+    $ wajig adduser fred
 
-If a number is provided then that many new users are created and
-the output will be username:password.
+    If a number is provided then that many new users are created and
+    the output will be username:password.
 
-  $ wajig adduser 5
-"""
+    $ wajig adduser 5"""
     number = args.number
     username = args.username
 
@@ -99,15 +116,15 @@ the output will be username:password.
             print(f"wajig adduser: error: file not accessible '{args.file}'.")
         else:
             usernames = []
-            for l in open(args.file, 'r'):
-                usernames.append(l.strip())
-    
+            for u in open(args.file, 'r'):
+                usernames.append(u.strip())
+
     elif number and re.match(r"^[0-9]+$", number):
         usernames = []
         for i in range(int(number)):
             code = ''.join(random.choice(string.ascii_lowercase) for _ in range(7))
             usernames.append(f"u{code}")
-            
+
     else:
         for u in username:
             if not re.match(r"^[a-z][-a-z0-9_]*$", u):
@@ -119,15 +136,15 @@ the output will be username:password.
     util.requires_package("pwgen")
     created = []
     for u in usernames:
-        command = f'adduser {u} --gecos "" --disabled-password'
-        perform.execute(command, root=True, teach=args.teach, noop=args.noop)
+        cmd = f'adduser {u} --gecos "" --disabled-password'
+        perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
         
-        command = f"pwgen -s -y 20 1"
-        password = perform.execute(command, pipe=True, teach=args.teach, noop=args.noop)
+        cmd = f"pwgen -s 20 1"
+        password = perform.execute(cmd, pipe=True, teach=args.teach, noop=args.noop)
         if password: password = password.readline().strip()
         
-        command = f'echo "{u}:{password}" | sudo chpasswd'
-        perform.execute(command, root=True, teach=args.teach, noop=args.noop)
+        cmd = f'echo "{u}:{password}" | sudo chpasswd'
+        perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
 
         print()
         created.append(f"{u}:{password}")
@@ -149,9 +166,9 @@ def autoalts(args):
 def autodownload(args):
     """Do an update followed by a download of all updated packages"""
     util.do_update(args.noop)
-    command = ("/usr/bin/apt-get --download-only --show-upgraded --assume-yes "
+    cmd = ("/usr/bin/apt-get --download-only --show-upgraded --assume-yes "
                "--force-yes dist-upgrade")
-    perform.execute(command, root=True)
+    perform.execute(cmd, root=True)
     if not args.noop:
         upgradable_packages = util.upgradable()
         if upgradable_packages:
@@ -179,16 +196,16 @@ def build(args):
     util.requires_package("sudo")
     # First make sure dependencies are met
     if not builddeps(args):
-        command = "apt-get {} source --build " + " ".join(args.packages)
-        command = command.format(args.noauth)
-        perform.execute(command)
+        cmd = "apt-get {} source --build " + " ".join(args.packages)
+        cmd = cmd.format(args.noauth)
+        perform.execute(cmd)
 
 
 def builddeps(args):
     """Install build-dependencies for given packages"""
-    command = "/usr/bin/apt-get {} {} build-dep " + " ".join(args.packages)
-    command = command.format(args.yes, args.noauth)
-    return perform.execute(command, root=True, log=True, teach=args.teach, noop=args.noop)
+    cmd = "/usr/bin/apt-get {} {} build-dep " + " ".join(args.packages)
+    cmd = cmd.format(args.yes, args.noauth)
+    return perform.execute(cmd, root=True, log=True, teach=args.teach, noop=args.noop)
 
 
 def changelog(args):
@@ -238,10 +255,10 @@ def changelog(args):
                 changelog += "{:=^79}\n".format(" local changelog ")
             f.write(changelog)
         if package.is_installed:
-            command = util.local_changelog(args.package, tmp)
-            if not command:
+            cmd = util.local_changelog(args.package, tmp)
+            if not cmd:
                 return
-            perform.execute(command)
+            perform.execute(cmd)
         with open(tmp) as f:
             for line in f:
                 try:
@@ -294,8 +311,8 @@ With a list of usernames, delete each user.
   $ wajig deluser fred susan
 """
     for u in args.username:
-        command = f"sudo deluser --remove-home --backup {u}"
-        perform.execute(command, root=True, teach=args.teach, noop=args.noop)
+        cmd = f"sudo deluser --remove-home --backup {u}"
+        perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
         if not u == args.username[-1]: print()
 
 
@@ -350,6 +367,22 @@ def describenew(args):
     util.newly_available()
 
 
+# DISABLE
+
+def disable(args):
+    """Disable user accounts.
+
+    The user account is retained and can be re-ENABLEd. The action is
+    to actually place a '!' in front of the encrypted password in the
+    password file.  With a list of usernames, diable each user.
+
+    $ wajig disable fred susan"""
+    for u in args.username:
+        cmd = f"sudo usermod --lock {u}"
+        perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
+        if not u == args.username[-1]: print()
+
+
 def distupgrade(args):
     """Comprehensive system upgrade
 
@@ -377,10 +410,10 @@ def distupgrade(args):
 def download(args):
     """Download one or more packages without installing them"""
     print("Packages being downloaded to /var/cache/apt/archives/")
-    command = "/usr/bin/apt-get --reinstall --download-only install "
+    cmd = "/usr/bin/apt-get --reinstall --download-only install "
     packages = util.consolidate_package_names(args)
-    command = command + " ".join(packages)
-    perform.execute(command, root=True, teach=args.teach, noop=args.noop)
+    cmd = cmd + " ".join(packages)
+    perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
 
 
 def editsources(args):
@@ -388,11 +421,28 @@ def editsources(args):
     perform.execute("/usr/bin/apt edit-source", root=True, teach=args.teach, noop=args.noop)
 
 
+# ENABLE
+
+def enable(args):
+    """Enable user accounts.
+
+    A user account that has been `disable`d is re-enabled. The action
+    is to actually remove a '!' in front of the encrypted password in
+    the password file.  With a list of usernames, enable each user.
+
+    $ wajig enable mary john"""
+    for u in args.username:
+        cmd = f"sudo usermod --unlock {u}"
+        perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
+        if not u == args.username[-1]:
+            print()
+
+
 def extract(args):
     """Extract the files from a package file to a directory"""
-    command = "dpkg --extract {} {}"
-    command = command.format(args.debfile, args.destination_directory)
-    perform.execute(command, teach=args.teach, noop=args.noop)
+    cmd = "dpkg --extract {} {}"
+    cmd = cmd.format(args.debfile, args.destination_directory)
+    perform.execute(cmd, teach=args.teach, noop=args.noop)
 
 
 def fixconfigure(args):
@@ -402,14 +452,14 @@ def fixconfigure(args):
 
 def fixinstall(args):
     """Fix an install interrupted by broken dependencies"""
-    command = "/usr/bin/apt-get --fix-broken {} install".format(args.noauth)
-    perform.execute(command, root=True, log=True, teach=args.teach, noop=args.noop)
+    cmd = "/usr/bin/apt-get --fix-broken {} install".format(args.noauth)
+    perform.execute(cmd, root=True, log=True, teach=args.teach, noop=args.noop)
 
 
 def fixmissing(args):
     """Fix and install even though there are missing dependencies"""
-    command = "/usr/bin/apt-get --ignore-missing {} upgrade".format(args.noauth)
-    perform.execute(command, root=True, log=True, teach=args.teach, noop=args.noop)
+    cmd = "/usr/bin/apt-get --ignore-missing {} upgrade".format(args.noauth)
+    perform.execute(cmd, root=True, log=True, teach=args.teach, noop=args.noop)
 
 
 def force(args):
@@ -420,16 +470,16 @@ def force(args):
           whatever reason
     """
 
-    command = "/usr/bin/dpkg --install --force overwrite --force depends "
+    cmd = "/usr/bin/dpkg --install --force overwrite --force depends "
     archives = "/var/cache/apt/archives/"
 
     # For a .deb file we simply force install it.
     if args.packages[0].endswith(".deb"):
         for package in args.packages:
             if os.path.exists(package):
-                command += "'" + package + "' "
+                cmd += "'" + package + "' "
             elif os.path.exists(archives + package):
-                command += "'" + archives + package + "' "
+                cmd += "'" + archives + package + "' "
             else:
                 message = ("File {} not found. "
                            "Searched current directory and {}."
@@ -457,9 +507,9 @@ def force(args):
                 debpkg = matches.readline().strip()
 
             # Force install the package from the download archive.
-            command += "'" + archives + debpkg + "' "
+            cmd += "'" + archives + debpkg + "' "
 
-    perform.execute(command, root=True, log=True, teach=args.teach, noop=args.noop)
+    perform.execute(cmd, root=True, log=True, teach=args.teach, noop=args.noop)
 
 
 def hold(args):
@@ -467,8 +517,8 @@ def hold(args):
     for package in args.packages:
         # The dpkg needs sudo but not the echo.
         # Do all of it as root then!
-        command = '/bin/echo "{} hold" | /usr/bin/dpkg --set-selections'
-        perform.execute(command.format(package), root=True, teach=args.teach, noop=args.noop)
+        cmd = '/bin/echo "{} hold" | /usr/bin/dpkg --set-selections'
+        perform.execute(cmd.format(package), root=True, teach=args.teach, noop=args.noop)
     print("The following packages are on hold:")
     perform.execute("dpkg --get-selections | grep -E 'hold$' | cut -f1", teach=args.teach, noop=args.noop)
 
@@ -627,17 +677,27 @@ def listfiles(args):
             print(line)
 
 
+def listgroups(args):
+    """List the unix groups defined for this computer.
+
+    Groups in unix are used for managing users and their permissions
+    to access commands and files."""
+    cmd = "cat /etc/group | cut -d':' -f1 | sort"
+    perform.execute(cmd, teach=args.teach, noop=args.noop)
+
+
 def listhold(args):
     """List packages that are on hold (i.e. those that won't be upgraded)"""
-    perform.execute("dpkg --get-selections | grep -E 'hold$' | cut -f1", teach=args.teach, noop=args.noop)
+    cmd = "dpkg --get-selections | grep -E 'hold$' | cut -f1"
+    perform.execute(cmd, teach=args.teach, noop=args.noop)
 
 
 def listinstalled(args):
     """List installed packages"""
-    command = "dpkg --get-selections | cut -f1"
+    cmd = "dpkg --get-selections | cut -f1"
     if args.pattern:
-        command += " | grep -E '{}' | sort -k 1b,1".format(args.pattern)
-    perform.execute(command, teach=args.teach, noop=args.noop)
+        cmd += " | grep -E '{}' | sort -k 1b,1".format(args.pattern)
+    perform.execute(cmd, teach=args.teach, noop=args.noop)
 
 
 def listlog(args):
@@ -654,10 +714,10 @@ def listnames(args):
 
 def listpackages(args):
     """List the status, version, and description of installed packages"""
-    command = "dpkg --list '*' | grep -E -v 'no description avail'"
+    cmd = "dpkg --list '*' | grep -E -v 'no description avail'"
     if args.pattern:
-        command += " | grep -E '{}' | sort -k 1b,1".format(args.pattern)
-    perform.execute(command, teach=args.teach, noop=args.noop)
+        cmd += " | grep -E '{}' | sort -k 1b,1".format(args.pattern)
+    perform.execute(cmd, teach=args.teach, noop=args.noop)
 
 
 def listscripts(args):
@@ -665,16 +725,16 @@ def listscripts(args):
     package = args.debfile
     scripts = ["preinst", "postinst", "prerm", "postrm"]
     if package.endswith(".deb"):
-        command = "ar p " + package + " control.tar.gz | tar ztvf -"
-        pkgScripts = perform.execute(command, pipe=True).readlines()
+        cmd = "ar p " + package + " control.tar.gz | tar ztvf -"
+        pkgScripts = perform.execute(cmd, pipe=True).readlines()
         for script in scripts:
             if "./" + script in "".join(pkgScripts):
                 nlen = int((72 - len(script)) / 2)
                 print(">"*nlen, script, "<"*nlen)
-                command = "ar p " + package + " control.tar.gz |" +\
+                cmd = "ar p " + package + " control.tar.gz |" +\
                           "tar zxvf - -O ./" + script +\
                           " 2>/dev/null"
-                perform.execute(command, teach=args.teach, noop=args.noop)
+                perform.execute(cmd, teach=args.teach, noop=args.noop)
     else:
         root = "/var/lib/dpkg/info/"
         for script in scripts:
@@ -711,21 +771,21 @@ def listsections(args):
 
 def liststatus(args):
     """Same as list but only prints first two columns, not truncated"""
-    command = "COLUMNS=400 "
-    command += "dpkg --list '*' | grep -E -v 'no description avail'"
-    command += " | awk '{print $1,$2}'"
+    cmd = "COLUMNS=400 "
+    cmd += "dpkg --list '*' | grep -E -v 'no description avail'"
+    cmd += " | awk '{print $1,$2}'"
     if args.pattern:
-        command += " | grep -E '{}' | sort -k 1b,1".format(args.pattern)
-    perform.execute(command, teach=args.teach, noop=args.noop)
+        cmd += " | grep -E '{}' | sort -k 1b,1".format(args.pattern)
+    perform.execute(cmd, teach=args.teach, noop=args.noop)
 
 
 def localupgrade(args):
     """Upgrade using only packages that are already downloaded"""
-    command = (
+    cmd = (
         "/usr/bin/apt-get --no-download --ignore-missing "
         "--show-upgraded upgrade"
     )
-    perform.execute(command, root=True, log=True, teach=args.teach, noop=args.noop)
+    perform.execute(cmd, root=True, log=True, teach=args.teach, noop=args.noop)
 
 
 def madison(args):
@@ -987,6 +1047,22 @@ def restart(args):
     command = "/usr/sbin/service {} restart".format(args.daemon)
     perform.execute(command, root=True, teach=args.teach, noop=args.noop)
 
+
+# RMGROUP
+
+def rmgroup(args):
+    """Remove a user from a group.
+
+    Unix groups are used to provide group access to commands or
+    files. For example, adding users to the group `sudo` give those
+    users super user access through the `sudo` command. This command
+    allows users to be removed from groups.
+
+    $ wajig rmgroup fred sudo"""
+    cmd = f'/usr/sbin/deluser {args.username} {args.group}'
+    perform.execute(cmd, root=True, teach=args.teach, noop=args.noop)
+
+
 # RMREPO
 
 def rmrepo(args):
